/*
    Project: TCGINA
    Version: V1.30
    File   : tc.h

    Copyright (c) 2005-2012 Author of TCGINA DLL. All rights reserved.
    Public Key Fingerprint: 294B A769 4A0A CC05 DAE6 00DD FF47 8C72 4097 67CE

    Covered by TrueCrypt License 2.3 the full text of which is contained
    in the file License.txt included in TCGINA binary and source code
    distribution archives.

    The copyright notice of this source code file and the public key
    fingerprint must be retained in full.
*/

#ifndef __TC_H_D47DF546F16B490bA6D8523603A5594D
#define __TC_H_D47DF546F16B490bA6D8523603A5594D

#ifdef _WIN32_WINNT
#  if _WIN32_WINNT < 0x0500
#    undef _WIN32_WINNT
#    define _WIN32_WINNT 0x0500
#    pragma message("WARNING: _WIN32_WINNT redefined to 0x0500")
#  endif
#else
#  define _WIN32_WINNT 0x0500
#endif

#include <windows.h>
#include <string.h>
#include "secmem.h"
#include "misc.h"

#ifdef __cplusplus
extern "C" {
#endif

/*------------------------------------------------------------------------------
// Debug functions                                                            */

void TC_DbgPrintfFuncA(LPCSTR, ...);
void TC_DbgPrintfFuncW(LPWSTR, ...);

#ifdef TC_DEBUG
   void  WINAPI TC_DbgSetHinstanceFunc(HINSTANCE);
   void  WINAPI TC_DbgPutsFuncA(LPCSTR);
   void  WINAPI TC_DbgPutsFuncW(LPCWSTR);
   void  WINAPI TC_AssertFunc(BOOL, LPCSTR, DWORD);
   PVOID WINAPI TC_AllocFunc(SIZE_T, LPCSTR, DWORD);
   PVOID WINAPI TC_CallocFunc(SIZE_T, LPCSTR, DWORD);
   void  WINAPI TC_FreeFunc(LPVOID, LPCSTR, DWORD);
   PVOID WINAPI TC_ConvertLocalAllocBlockFunc(PVOID, SIZE_T, LPCSTR, DWORD);
#  define TC_DbgSetHInstance(h) TC_DbgSetHinstanceFunc(h);
#  define TC_DbgPrintfA  TC_DbgPrintfFuncA
#  define TC_DbgPrintfW  TC_DbgPrintfFuncW
#  define TC_DbgPutsA(x) TC_DbgPutsFuncA(x)
#  define TC_DbgPutsW(x) TC_DbgPutsFuncW(x)
#  define TC_Assert(x)   TC_AssertFunc((x)?TRUE:FALSE, __FILE__, __LINE__)
#  define TC_Alloc(n)    TC_AllocFunc(n, __FILE__, __LINE__)
#  define TC_Calloc(n)   TC_CallocFunc(n, __FILE__, __LINE__)
#  define TC_Free(p)     TC_FreeFunc((PVOID)p, __FILE__, __LINE__)
#  define TC_ConvertLocalAllocBlock(p,n) \
     TC_ConvertLocalAllocBlockFunc(p, n, __FILE__, __LINE__)
#else
#  define TC_DbgSetHInstance(h)
#  define TC_DbgPrintfA  __noop
#  define TC_DbgPrintfW  __noop
#  define TC_DbgPutsA(x)
#  define TC_DbgPutsW(x)
#  define TC_Assert(x)
#  define TC_Alloc(n)  LocalAlloc(LMEM_FIXED, n)
#  define TC_Calloc(n) LocalAlloc(LMEM_FIXED|LMEM_ZEROINIT, n)
#  define TC_Free(p)   LocalFree((PVOID)p)
#  define TC_ConvertLocalAllocBlock(p, n) (PVOID)(p)
#endif

#ifdef UNICODE
#  define TC_DbgPrintf  TC_DbgPrintfW
#  define TC_DbgPuts(x) TC_DbgPutsW(x)
#else
#  define TC_DbgPrintf  TC_DbgPrintfA
#  define TC_DbgPuts(x) TC_DbgPutsA(x)
#endif


/*------------------------------------------------------------------------------
// Auxiliary functions                                                        */

#ifdef _WIN64
#  define TC_SetWindowLongPtr(hWnd, nIndex, dwNewLong) \
     SetWindowLongPtr(hWnd, nIndex, dwNewLong)
#  define TC_GetWindowLongPtr(hWnd, nIndex) \
     GetWindowLongPtr(hWnd, nIndex)
#else
#  define TC_SetWindowLongPtr(hWnd, nIndex, dwNewLong) \
     SetWindowLongPtr(hWnd, nIndex, PtrToLong(dwNewLong))
#  define TC_GetWindowLongPtr(hWnd, nIndex) \
     LongToPtr(GetWindowLongPtr(hWnd, nIndex))
#endif

#define TC_ZeroMemory(p, n) memset(p, '\0', n)
#define TC_WipeMemory(p, n) sec_memset(p, '\xff', n); sec_memset(p, '\0', n)
BOOL WINAPI TC_AppendString(LPTSTR pszDest, DWORD nSize, LPCTSTR pszSrc);
BOOL WINAPI TC_CopyString(LPTSTR pszDest, DWORD nSize, LPCTSTR pszSrc);

#define TC_ConvertUnicodeToAnsi(paszDest, cbSize, pwszSrc) \
  WideCharToMultiByte(CP_ACP, 0, pwszSrc, -1, paszDest, cbSize, NULL, NULL)
#define TC_ConvertAnsiToUnicode(pwszDest, nSize, paszSrc) \
  MultiByteToWideChar(CP_ACP, 0, paszSrc, -1, pwszDest, nSize)
#define TC_ConvertUnicodeToUTF8(paszDest, cbSize, pwszSrc) \
  WideCharToMultiByte(CP_UTF8, 0, pwszSrc, -1, paszDest, cbSize, NULL, NULL)
#define TC_ConvertUTF8ToUnicode(pwszDest, nSize, paszSrc) \
  MultiByteToWideChar(CP_UTF8, 0, paszSrc, -1, pwszDest, nSize)

#ifdef UNICODE
#  define TC_CopyStringToAnsi(paszDest, nSize, pszSrc) \
     TC_ConvertUnicodeToAnsi(paszDest, nSize, pszSrc)
#  define TC_CopyStringToUnicode(pwszDest, nSize, pszSrc) \
     TC_CopyString(pwszDest, nSize, pszSrc)
#  define TC_CopyStringFromAnsi(pszDest, nSize, paszSrc) \
     TC_ConvertAnsiToUnicode(pszDest, nSize, paszSrc)
#  define TC_CopyStringFromUnicode(pszDest, nSize, pwszSrc) \
     TC_CopyString(pszDest, nSize, pwszSrc)
#  define TC_CopyStringToUTF8(paszDest, nSize, pszSrc) \
     TC_ConvertUnicodeToUTF8(paszDest, nSize, pszSrc)
#else
#  define TC_CopyStringToAnsi(paszDest, nSize, pszSrc) \
     TC_CopyString(paszDest, nSize, pszSrc)
#  define TC_CopyStringToUnicode(pwszDest, nSize, pszSrc) \
     TC_ConvertAnsiToUnicode(pwszDest, nSize, pszSrc)
#  define TC_CopyStringFromAnsi(pszDest, nSize, paszSrc) \
     TC_CopyString(pszDest, nSize, paszSrc)
#  define TC_CopyStringFromUnicode(pszDest, nSize, pwszSrc) \
     TC_ConvertUnicodeToAnsi(pszDest, nSize, pwszSrc)
#endif

void WINAPI TC_TrimString(LPTSTR pszString);
BOOL WINAPI TC_ReplaceString(LPTSTR pszDest, DWORD nDestSize,
  LPCTSTR pszSearch, LPCTSTR pszReplace);
LPCTSTR WINAPI TC_GetNextPathName(LPCTSTR pszSearchPath, LPTSTR pszDest,
  DWORD nDestSize);

/* Return value must be freed with TC_Free() */
LPCTSTR WINAPI TC_LoadRegistryString(HKEY hKey, LPCTSTR pszValueName);

BOOL WINAPI TC_ReadRegistryString(HKEY hKey, LPCTSTR pszValueName,
  LPTSTR pszDest, DWORD nSize);
BOOL WINAPI TC_GetRegistryString(HKEY hRootKey, LPCTSTR pszKeyName,
  LPCTSTR pszValueName, LPTSTR pszDest, DWORD nSize);

/* Return value must be freed with TC_Free() */
LPCTSTR WINAPI TC_LoadRegistryMultiString(HKEY hKey, LPCTSTR pszValueName,
  PDWORD pdwType);

BOOL WINAPI TC_ReadRegistryMultiString(HKEY hKey, LPCTSTR pszValueName,
  LPTSTR pszDest, DWORD nSize, PDWORD pdwType);
BOOL WINAPI TC_GetRegistryMultiString(HKEY hRootKey, LPCTSTR pszKeyName,
  LPCTSTR pszValueName, LPTSTR pszDest, DWORD nSize, PDWORD pdwType);
BOOL WINAPI TC_ReadRegistryStringList(HKEY hKey, LPCTSTR pszValueName,
  LPTSTR pszDest, DWORD nSize);
BOOL WINAPI TC_GetRegistryStringList(HKEY hRootKey, LPCTSTR pszKeyName,
  LPCTSTR pszValueName, LPTSTR pszDest, DWORD nSize);

BOOL WINAPI TC_ReadRegistryInteger(HKEY hKey, LPCTSTR pszValueName,
  DWORD *pdwValueData);
DWORD WINAPI TC_GetRegistryInteger(HKEY hRootKey, LPCTSTR pszKeyName,
  LPCTSTR pszValueName, DWORD dwValueData);

/* Return value must be freed with TC_Free() */
LPCTSTR WINAPI TC_GetStringSid(LPCTSTR pszUserName);
/* Return value must be freed with TC_Free() */
LPCTSTR WINAPI TC_GetUserName(LPCTSTR pszSid);
/* Return value must be freed with TC_Free() */
LPCTSTR WINAPI TC_GetProfileStringSid(LPCTSTR pszUserName);

BOOL WINAPI TC_FileExists(LPCTSTR pszFileName);
BOOL WINAPI TC_PathExists(LPCTSTR pszFileName);
BOOL WINAPI TC_PathIsMountPoint(LPCTSTR pszFileName);
UINT WINAPI TC_GetDriveType(TCHAR chDriveLetter);

void WINAPI TC_RemoveTerminatingBackslash(LPTSTR pszPath);
BOOL WINAPI TC_GetProgramFilesDirectory(LPTSTR pszProgramFilesDirectory,
  DWORD nSize);
BOOL WINAPI TC_GetProfilesDirectory(LPTSTR pszProfilesDirectory, DWORD nSize);

BOOL WINAPI TC_VolumeIsAccessable(TCHAR chDriveLetter);
BOOL WINAPI TC_DriveNameIsValid(LPCTSTR pszDrive);
BOOL WINAPI TC_DriveLetterIsFree(TCHAR chDriveLetter);
BOOL WINAPI TC_NormalizeFileName(LPTSTR pszDest, DWORD nDestSize,
  LPCTSTR pszSrc);

BOOL WINAPI TC_VolumeHostExists(LPCTSTR pszHostName);

BOOL WINAPI TC_UNCFileName(LPCTSTR pszFileName);
BOOL WINAPI TC_RemoteFile(LPCTSTR pszFileName);
DWORD WINAPI TC_GetDriveNo(TCHAR chDriveLetter);
DWORD WINAPI TC_GetDriveBitMask(TCHAR chDriveLetter);

/*------------------------------------------------------------------------------
// Partition data                                                             */

typedef struct
{
    DWORD dwDisk;
    DWORD dwPartition;

    /* for internal use only */
    DWORD dwPartitionCount;

} TC_PARTITION_DATA , *PTC_PARTITION_DATA;

BOOL WINAPI TC_GetFirstPartition(PTC_PARTITION_DATA pa);
BOOL WINAPI TC_GetNextPartition(PTC_PARTITION_DATA pa);
BOOL WINAPI TC_GetPartitionDeviceName(const PTC_PARTITION_DATA pa,
  LPTSTR pszDest, DWORD nSize);
BOOL WINAPI TC_GetPartitionData(PTC_PARTITION_DATA pd,
  LPCTSTR pszPartitionName);

/*------------------------------------------------------------------------------
// TC API                                                                     */

#define TC_MAX_PASSWORD_LEN   64
#define TC_MAX_PASSWORD       (TC_MAX_PASSWORD_LEN + 1)

typedef PVOID TC_HANDLE;

#define TC_DEFAULT_SETTINGS   0x00000000
#define TC_EXE_INTERFACE      0x00010000
#define TC_SYS_INTERFACE      0x00020000
#define TC_ALL_INTERFACES     0x00030000
/* Return value must be freed with TC_Close() */
TC_HANDLE WINAPI TC_Open(DWORD dwFlags);

void WINAPI TC_Close(TC_HANDLE hTC);

#define TC_CACHE_PASSWORDS    0x00000001
#define TC_READ_ONLY          0x00000002
#define TC_REMOVABLE          0x00000004
#define TC_SHARED             0x00000008
#define TC_UPDATE_TIMESTAMPS  0x00000010
#define TC_PERSISTENT         0x00000020
#define TC_SYSTEM             0x00000040
BOOL WINAPI TC_Mount(TC_HANDLE hTC, LPCTSTR pszTCFileName,
  LPCTSTR pszPassword, LPCTSTR pszDrive, DWORD dwFlags,
  LPCTSTR pszKeyFiles, BOOL fProtectHiddenVolume,
  LPCTSTR pszHiddenVolumePassword, LPCTSTR pszHiddenVolumeKeyFiles);
BOOL WINAPI TC_MountWithAnsiPassword(TC_HANDLE hTC, LPCTSTR pszTCFileName,
  LPCSTR paszPassword, LPCTSTR pszDrive, DWORD dwFlags,
  LPCTSTR pszKeyFiles, BOOL fProtectHiddenVolume,
  LPCSTR paszHiddenVolumePassword, LPCTSTR pszHiddenVolumeKeyFiles);

BOOL WINAPI TC_WipeCache(TC_HANDLE hTC);

#define TC_WIPE_CACHE        0x00000010
#define TC_DISMOUNT_FORCIBLY 0x00000020
BOOL WINAPI TC_Dismount(TC_HANDLE hTC, LPCTSTR pszDrive, DWORD dwFlags);
BOOL WINAPI TC_GetVolumeFileName(TC_HANDLE hTC, LPCTSTR pszDrive,
  LPTSTR pszDest, DWORD nSize);

#define TCMD_DISMOUNTABLE 1
#define TCMD_PERSISTENT   2
#define TCMD_SYSTEM       4
#define TCMD_ALL          (TCMD_DISMOUNTABLE | TCMD_PERSISTENT | TCMD_SYSTEM)
BOOL WINAPI TC_GetMountedDrives(TC_HANDLE hTC, PDWORD pdwDrives, DWORD dwFlags);
VOID WINAPI TC_RefreshDriveLetters(TC_HANDLE hTC);

/* TC API error codes */
DWORD WINAPI TC_GetLastError(TC_HANDLE hTC);
BOOL WINAPI TC_SetLastError(TC_HANDLE hTC, DWORD dwErrorCode);
BOOL WINAPI TC_GetErrorString(DWORD dwErrorCode, LPTSTR pszText,
  DWORD nSize);

#define TC_E(e)            ((DWORD)(e))
#define TC_ERROR(f,c)      (0xe8008000 + ((f)*0x10000) + (c))
#define TC_WIN32_ERROR(e)  ((TC_E(e) & 0x20000000) ? ERROR_TC_CUSTOM : TC_E(e))
#define TC_DRIVER_ERROR(e) ((TC_E(e) <= 0xffff) ? TC_ERROR(1, TC_E(e)) : TC_ERROR(1, 0))

#define ERROR_TC_GENERIC                    TC_ERROR(0,  1)
#define ERROR_TC_CUSTOM                     TC_ERROR(0,  2)
#define ERROR_TC_INVALID_POINTER            TC_ERROR(0,  3)
#define ERROR_TC_INVALID_HANDLE             TC_ERROR(0,  4)
#define ERROR_TC_INVALID_PASSWORD           TC_ERROR(0,  5)
#define ERROR_TC_INVALID_FILENAME           TC_ERROR(0,  6)
#define ERROR_TC_INVALID_DRIVE_LETTER       TC_ERROR(0,  7)
#define ERROR_TC_DRIVE_LETTER_NOT_FREE      TC_ERROR(0,  8)
#define ERROR_TC_PARTITION_ALREADY_MOUNTED  TC_ERROR(0,  9)
#define ERROR_TC_FILE_ALREADY_MOUNTED       TC_ERROR(0, 10)
#define ERROR_TC_FILE_NOT_FOUND             TC_ERROR(0, 11)
#define ERROR_TC_OUT_OF_SECURE_MEMORY       TC_ERROR(0, 12)
#define ERROR_TC_INVALID_KEYFILES           TC_ERROR(0, 13)
#define ERROR_TC_CANNOT_FIND_MOUNTED_VOLUME TC_ERROR(0, 14)
#define ERROR_TC_CANNOT_CREATE_DOS_DEVICE   TC_ERROR(0, 15)
#define ERROR_TC_INVALID_PROFILE_IMAGE_PATH TC_ERROR(0, 16)

/* Device driver errors */
#define ERROR_TC_WRONG_PASSWORD             TC_ERROR(1, 3)

#ifdef __cplusplus
}
#endif

#endif
