/*
    Project: TCGINA
    Version: V1.30
    File   : setup.c

    Copyright (c) 2005-2012 Author of TCGINA DLL. All rights reserved.
    Public Key Fingerprint: 294B A769 4A0A CC05 DAE6 00DD FF47 8C72 4097 67CE

    Covered by TrueCrypt License 2.3 the full text of which is contained
    in the file License.txt included in TCGINA binary and source code
    distribution archives.

    The copyright notice of this source code file and the public key
    fingerprint must be retained in full.
*/

#include <windows.h>

#ifdef _MSC_VER
#  pragma warning( push )
#  pragma warning( disable : 4201 )
#endif
#include <shlobj.h>
#ifdef _MSC_VER
#  pragma warning( pop )
#endif

#include <tchar.h>
#include <tc.h>
#include "setupdlg.h"

#pragma warning(push)
#pragma warning(disable:4996)

BOOL IsTcginaLog(PFILE_INFO pfi)
{
    BOOL fResult = FALSE;

    if (pfi->fd.nFileSizeHigh == 0 && pfi->fd.nFileSizeLow != 0 &&
      pfi->fd.nFileSizeLow < 0x2000000) {
        HANDLE hFile;
        char *pData = calloc(pfi->fd.nFileSizeLow + 1, 1);
        if (pData) {
            hFile = CreateFile(pfi->fd.cFileName, GENERIC_READ,
              FILE_SHARE_READ, NULL, OPEN_EXISTING, 0, NULL);
            if (hFile != INVALID_HANDLE_VALUE) {
                DWORD dwFileSize;
                if (ReadFile(hFile, pData, pfi->fd.nFileSizeLow, &dwFileSize,
                  NULL) && pfi->fd.nFileSizeLow == dwFileSize) {
                    strupr(pData);
                    if (strstr(pData, "TCGINA") != NULL &&
                      strstr(pData, "WLXNEGOTIATE") != NULL) {
                        fResult = TRUE;
                    }
                }
                CloseHandle(hFile);
            }
            free(pData);
        }
    }

    return fResult;
}

#pragma warning(pop)

#ifndef _WIN64
typedef BOOL (WINAPI *PFISWOW64PROCESS) (HANDLE hProcess, PBOOL pfWow64Process);
BOOL IsWow64(void)
{
    BOOL fIsWow64 = FALSE;
    PFISWOW64PROCESS pfIsWow64Process = (PFISWOW64PROCESS) GetProcAddress(
      GetModuleHandle(TEXT("KERNEL32")), "IsWow64Process");

    if (pfIsWow64Process != NULL) {
        pfIsWow64Process(GetCurrentProcess(), &fIsWow64);
    }

    return fIsWow64;
}
#endif

static BOOL AddFileInfo(PFILE_INFO_LIST *ppfil, PFILE_INFO pfi)
{
    PFILE_INFO_LIST pfil;

    if (*ppfil == NULL) {
        pfil = *ppfil = malloc(sizeof(FILE_INFO_LIST));
    } else {
        for (pfil = *ppfil; pfil->pNextItem; pfil = pfil->pNextItem) ;
        pfil = pfil->pNextItem = malloc(sizeof(FILE_INFO_LIST));
    }
    if (pfil == NULL) {
        MessageBox(NULL,
          TEXT("Out of memory error!"),
          TEXT("TCGINA Setup"),
          MB_OK|MB_ICONSTOP);
        return FALSE;
    }
    pfil->pNextItem = NULL;
    pfil->fi = *pfi;

    return TRUE;
}

static BOOL TerminatePath(LPTSTR pszPathName, DWORD cchPath)
{
    size_t cchStr;

    cchStr = _tcslen(pszPathName);
    if (cchStr != 0) {
        TCHAR ch = pszPathName[cchStr - 1];
        if (ch == TEXT('\\') || ch == TEXT('/')) {
            return TRUE;
        }
    }
    if (cchStr+1 >= cchPath) {
        return FALSE;
    }
    pszPathName[cchStr] = TEXT('\\');
    pszPathName[cchStr + 1] = TEXT('\0');

    return TRUE;
}

int WINAPI WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance,
  LPSTR lpCmdLine, int nCmdShow)
{
    OSVERSIONINFO     osvi;
    LPWSTR            *wszArgList;
    int               nArgs;
    TCHAR             szFileName[MAX_PATH];
    TCHAR             szFullPathName[MAX_PATH];
    LPTSTR            pszFilePart;
    HANDLE            hFile;
    WIN32_FIND_DATA   fd;
    SETUP_DIALOG_DATA sdd;
    FILE_INFO         fi;

    UNREFERENCED_PARAMETER(hInstance);
    UNREFERENCED_PARAMETER(hPrevInstance);
    UNREFERENCED_PARAMETER(lpCmdLine);
    UNREFERENCED_PARAMETER(nCmdShow);

    memset(&sdd, '\0', sizeof(sdd));

    osvi.dwOSVersionInfoSize = sizeof(osvi);
    if (!GetVersionEx(&osvi) || osvi.dwPlatformId != VER_PLATFORM_WIN32_NT ||
      osvi.dwMajorVersion != 5
#ifndef _WIN64
      || IsWow64()
#endif
      ) {
        MessageBox(NULL, TEXT("Windows Version not supported!"),
          TEXT("TCGINA Setup"), MB_OK|MB_ICONSTOP);
        return 1;
    }

    if (!IsUserAnAdmin()) {
        MessageBox(NULL, TEXT("Administrator privileges are required!"),
          TEXT("TCGINA Setup"), MB_OK|MB_ICONSTOP);
        return 1;
    }

    wszArgList = CommandLineToArgvW(GetCommandLineW(), &nArgs);
    if (wszArgList) {
        LocalFree(wszArgList);
    }

    /* TrueCrypt driver */
    GetSystemDirectory(szFileName, MAX_PATH);
    TC_AppendString(szFileName, countof(szFileName),
      TEXT("\\drivers\\truecrypt.sys"));
#ifdef _WIN64
    if (!FileExists(szFileName)) {
        GetWindowsDirectory(szFileName, MAX_PATH);
        TC_AppendString(szFileName, countof(szFileName),
          TEXT("\\SysWOW64\\Drivers\\truecrypt.sys"));
    }
#endif
    GetFileInfo(&sdd.fiTrueCrypt, szFileName);
    if (sdd.fiTrueCrypt.fExists) {
        hFile = CreateFile(TEXT("\\\\.\\TrueCrypt"), 0, 0, NULL,
          OPEN_EXISTING, 0, NULL);
        if (hFile != INVALID_HANDLE_VALUE) {
            sdd.fiTrueCrypt.fInUse = TRUE;
            CloseHandle(hFile);
        } else {
            sdd.fiTrueCrypt.fInUse = GetLastError() == ERROR_ACCESS_DENIED;
        }
    }

    /* GINA DLL */
    *szFileName = TEXT('\0');
    TC_GetRegistryString(HKEY_LOCAL_MACHINE, WINLOGON_KEY, TEXT("GinaDLL"),
      szFileName, MAX_PATH);
    if (*szFileName) {
        SearchPath(NULL, szFileName, TEXT(".DLL"), MAX_PATH,
          szFullPathName, &pszFilePart);
        GetFileInfo(&sdd.fiGinaDLL, szFullPathName);
        sdd.fGinaDLLIsTcginaDLL = IsTcginaDLL(&sdd.fiGinaDLL);
    }

    /* TCGINA Log Files */
    _tcscpy(szFileName, TEXT("C:\\TC.LOG"));
    for (*szFileName = TEXT('A'); *szFileName <= TEXT('Z'); (*szFileName)++) {
        if (TC_GetDriveType(*szFileName) == DRIVE_FIXED) {
            GetFileInfo(&fi, szFileName);
            if (IsTcginaLog(&fi)) {
                if (!AddFileInfo(&sdd.pfilFirstTcginaLog, &fi)) {
                    return 1;
                }
            }

        }
    }
    *szFileName = TEXT('\0');
    GetEnvironmentVariable(TEXT("SystemDrive"), szFileName, MAX_PATH);
    if (*szFileName) {
        TerminatePath(szFileName, MAX_PATH);
        if (TC_AppendString(szFileName, MAX_PATH, TEXT("TC.LOG"))) {
            GetFileInfo(&fi, szFileName);
            if (IsTcginaLog(&fi)) {
                if (!AddFileInfo(&sdd.pfilFirstTcginaLog, &fi)) {
                    return 1;
                }
            }
        }
    }
    *szFileName = TEXT('\0');
    GetEnvironmentVariable(TEXT("SystemRoot"), szFileName, MAX_PATH);
    if (*szFileName) {
        TerminatePath(szFileName, MAX_PATH);
        if (TC_AppendString(szFileName, MAX_PATH, TEXT("TC.LOG"))) {
            GetFileInfo(&fi, szFileName);
            if (IsTcginaLog(&fi)) {
                if (!AddFileInfo(&sdd.pfilFirstTcginaLog, &fi)) {
                    return 1;
                }
            }
        }
    }
    *szFileName = TEXT('\0');
    TC_GetRegistryString(HKEY_LOCAL_MACHINE, TCGINA_KEY,
      TEXT("LogFileName"), szFileName, MAX_PATH);
    GetFileInfo(&fi, szFileName);
    if (IsTcginaLog(&fi)) {
        if (!AddFileInfo(&sdd.pfilFirstTcginaLog, &fi)) {
            return 1;
        }
    }

    /* TCGINA DLLs */
    GetSystemDirectory(szFileName, MAX_PATH);
    TC_AppendString(szFileName, countof(szFileName), TEXT("\\TCGINA*.DLL"));
    hFile = FindFirstFile(szFileName, &fd);
    if (hFile != INVALID_HANDLE_VALUE) {
        do {
            FILE_INFO fi;
            GetSystemDirectory(szFileName, MAX_PATH);
            TC_AppendString(szFileName, countof(szFileName), TEXT("\\"));
            TC_AppendString(szFileName, countof(szFileName), fd.cFileName);
            if (GetFileInfo(&fi, szFileName)) {
                if (IsTcginaDLL(&fi)) {
                    if (!AddFileInfo(&sdd.pfilFirstTcginaDLL, &fi)) {
                        return 1;
                    }
                }
            }
        } while (FindNextFile(hFile, &fd));
        FindClose(hFile);
    }

    if (RegKeyExists(HKEY_LOCAL_MACHINE, TCGINA_ROOT_KEY)) {
        if (RegKeyIsEmpty(HKEY_LOCAL_MACHINE, TCGINA_ROOT_KEY)) {
            sdd.fRegistryKeyExists = TRUE;
        } else if (RegKeyExists(HKEY_LOCAL_MACHINE, TCGINA_KEY)) {
            sdd.fRegistryKeyExists = TRUE;
        }
    }

    if (sdd.fRegistryKeyExists || sdd.pfilFirstTcginaLog != NULL ||
      sdd.fGinaDLLIsTcginaDLL || sdd.pfilFirstTcginaDLL != NULL) {
        if (sdd.fGinaDLLIsTcginaDLL) {
            sdd.iStatus = sdd.fiGinaDLL.fInUse ? TCGS_RUNNING :
              TCGS_INSTALLED;
            if (*sdd.fiGinaDLL.szInfo) {
                TCHAR szConfiguration[80];
                LPTSTR pszVersion = *sdd.fiGinaDLL.szFileVersion ?
                  sdd.fiGinaDLL.szFileVersion : sdd.fiGinaDLL.szVersion;
                GetInfoItem(sdd.fiGinaDLL.szInfo, 2, szConfiguration, 80);
                _sntprintf(sdd.szTcginaVersion, countof(sdd.szTcginaVersion),
                  TEXT("%s V%s %s"), sdd.fiGinaDLL.szProductName,
                  pszVersion, szConfiguration);
            } else {
                _sntprintf(sdd.szTcginaVersion, countof(sdd.szTcginaVersion),
                  TEXT("%s V%s"), sdd.fiGinaDLL.szProductName,
                    sdd.fiGinaDLL.szFileVersion);
            }
        } else {
            sdd.iStatus = TCGS_PASSIVE;
        }
    }
    sdd.dwNoLMHash = TC_GetRegistryInteger(HKEY_LOCAL_MACHINE, LSA_KEY,
      TEXT("NoLMHash"), 0);

    sdd.pui = GetUserInfo();

    GetPrivateProfileString(TEXT("Setup"), TEXT("Caption"), TEXT(""),
      sdd.szCaption, countof(sdd.szCaption), GetIniFileName());

    DialogBoxParam(hInstance, MAKEINTRESOURCE(IDD_SETUP), NULL,
      SetupDialogProc, (LPARAM)&sdd);

    return 0;
}