/*
    Project: TCGINA
    Version: V1.30
    File   : hook.c

    Copyright (c) 2005-2012 Author of TCGINA DLL. All rights reserved.
    Public Key Fingerprint: 294B A769 4A0A CC05 DAE6 00DD FF47 8C72 4097 67CE

    Covered by TrueCrypt License 2.3 the full text of which is contained
    in the file License.txt included in TCGINA binary and source code
    distribution archives.

    The copyright notice of this source code file and the public key
    fingerprint must be retained in full.
*/

#include <windows.h>
#include <winnt.h>

#include "tc.h"
#define DBG_PRINTF TC_DbgPrintfA

static LONG (APIENTRY *OrgRegQueryValueExW)(HKEY hKey, LPCWSTR lpValueName,
  LPDWORD lpReserved, LPDWORD lpType, LPBYTE lpData, LPDWORD lpcbData) = NULL;

static LONG APIENTRY NewRegQueryValueExW(HKEY hKey, LPCWSTR lpValueName,
  LPDWORD lpReserved, LPDWORD lpType, LPBYTE lpData, LPDWORD lpcbData)
{
    if (OrgRegQueryValueExW == NULL) {
        return ERROR_INVALID_FUNCTION;
    }
    if (lpValueName != NULL && wcscmp(lpValueName, L"GinaDLL") == 0) {
        return ERROR_FILE_NOT_FOUND;
    }
    return OrgRegQueryValueExW(hKey, lpValueName, lpReserved, lpType,
      lpData, lpcbData);
}

static size_t *GetIATEntry(HINSTANCE hModule, LPCSTR pszProcName)
{
    size_t                   nBase;
    PIMAGE_DOS_HEADER        pIDH;
    PIMAGE_NT_HEADERS        pINH;
    PIMAGE_IMPORT_DESCRIPTOR pIID;
    PIMAGE_THUNK_DATA        pITD;
    PIMAGE_THUNK_DATA        pITD2;
    PIMAGE_IMPORT_BY_NAME    pIIBN;

    if (hModule == NULL) {
        hModule = GetModuleHandle(NULL);
    }
    nBase = (size_t)hModule;
    pIDH  = (PIMAGE_DOS_HEADER)nBase;
    pINH  = (PIMAGE_NT_HEADERS)(nBase + pIDH->e_lfanew);
    if (pIDH->e_magic != IMAGE_DOS_SIGNATURE ||
      pINH->OptionalHeader.Magic != IMAGE_NT_OPTIONAL_HDR_MAGIC ||
      pINH->OptionalHeader.NumberOfRvaAndSizes <=
      IMAGE_DIRECTORY_ENTRY_IMPORT) {
        return NULL;
    }
    pIID = (PIMAGE_IMPORT_DESCRIPTOR)(nBase + pINH->OptionalHeader.
      DataDirectory[IMAGE_DIRECTORY_ENTRY_IMPORT].VirtualAddress);
    for (; pIID->Characteristics != 0; pIID++) {
        pITD  = (PIMAGE_THUNK_DATA)(nBase + pIID->OriginalFirstThunk);
        pITD2 = (PIMAGE_THUNK_DATA)(nBase + pIID->FirstThunk);
        for (; pITD->u1.Ordinal != 0; pITD++, pITD2++) {
            if (!IMAGE_SNAP_BY_ORDINAL(pITD->u1.Ordinal)) {
                pIIBN = (PIMAGE_IMPORT_BY_NAME)(nBase + pITD->u1.Function);
                if (strcmp(pszProcName, pIIBN->Name) == 0) {
                    return (size_t *)pITD2;
                }
            }
        }
    }

    return NULL;
}

BOOL InstallHook(HMODULE hTargetDll, HMODULE hHookDll)
{
    OSVERSIONINFO osvi;
    HANDLE        hCurrProcess;
    DWORD         dwOldProtect;
    size_t        *pIATEntry;
    BOOL          fResult;

    static BOOL   fHookAlreadyInstalled = FALSE;

    if (fHookAlreadyInstalled) {
        DBG_PRINTF("Hook is already installed.\n");
        return TRUE;
    }

    pIATEntry = GetIATEntry(hTargetDll, "RegQueryValueExW");
    if (pIATEntry == NULL) {
        DBG_PRINTF("Error: Cannot find IAT entry!\n");
        return FALSE;
    }

    hCurrProcess = GetCurrentProcess();
    fResult      = TRUE;

    DBG_PRINTF("Begin of RegQueryValueExW hooking\n");

    osvi.dwOSVersionInfoSize = sizeof(OSVERSIONINFO);
    if (!GetVersionEx(&osvi)) {
        DBG_PRINTF("Error: Cannot query OS version!\n");
        return FALSE;
    }
    if (osvi.dwPlatformId != VER_PLATFORM_WIN32_NT ||
      osvi.dwMajorVersion != 5 || osvi.dwMinorVersion < 1) {
        DBG_PRINTF("Error: Unsupported OS version!\n");
        return FALSE;
    }

    if (!VirtualProtectEx(hCurrProcess, pIATEntry, sizeof(size_t),
      PAGE_EXECUTE_WRITECOPY, &dwOldProtect)) {
        DBG_PRINTF("Error: Cannot unprotect IAT entry!\n");
        return FALSE;
    }
    __try {
        *(size_t *) &OrgRegQueryValueExW = *pIATEntry;
        *pIATEntry = (size_t)NewRegQueryValueExW;
    } __except(EXCEPTION_EXECUTE_HANDLER) {
        DBG_PRINTF("Exception %u!\n", (int)GetExceptionCode());
        fResult = FALSE;
    }
    VirtualProtectEx(hCurrProcess, pIATEntry, sizeof(size_t), dwOldProtect,
      NULL);
    FlushInstructionCache(hCurrProcess, pIATEntry, sizeof(size_t));

    if (!fResult) {
        return FALSE;
    }

    if (hHookDll != NULL) {
        TCHAR szDllFileName[MAX_PATH];
        if (GetModuleFileName(hHookDll, szDllFileName,
          countof(szDllFileName)) != 0) {
            if (LoadLibrary(szDllFileName) == NULL) {
                DBG_PRINTF("Warning: Cannot increment DLL reference count!\n");
            }
        } else {
            DBG_PRINTF("Warning: Cannot get module filename!\n");
        }
    }

    DBG_PRINTF("End of RegQueryValueExW hooking\n");

    fHookAlreadyInstalled = TRUE;

    return TRUE;
}
